@@ -15,6 +15,7 @@
 
 #include <ext/standard/url.h>
 
+/* php_http_url_mod() */
 #define PHP_HTTP_URL_REPLACE		0x000
 #define PHP_HTTP_URL_JOIN_PATH		0x001
 #define PHP_HTTP_URL_JOIN_QUERY		0x002
@@ -57,42 +58,22 @@ typedef struct php_http_url {
 } php_http_url_t;
 
 PHP_HTTP_API php_http_url_t *php_http_url_parse(const char *str, size_t len, unsigned flags TSRMLS_DC);
-/* deprecated */
-PHP_HTTP_API void php_http_url(int flags, const php_url *old_url, const php_url *new_url, php_url **url_ptr, char **url_str, size_t *url_len TSRMLS_DC);
-/* use this instead */
+PHP_HTTP_API php_http_url_t *php_http_url_parse_authority(const char *str, size_t len, unsigned flags TSRMLS_DC);
 PHP_HTTP_API php_http_url_t *php_http_url_mod(const php_http_url_t *old_url, const php_http_url_t *new_url, unsigned flags TSRMLS_DC);
 PHP_HTTP_API php_http_url_t *php_http_url_copy(const php_http_url_t *url, zend_bool persistent);
 PHP_HTTP_API php_http_url_t *php_http_url_from_struct(HashTable *ht);
 PHP_HTTP_API php_http_url_t *php_http_url_from_zval(zval *value, unsigned flags TSRMLS_DC);
 PHP_HTTP_API HashTable *php_http_url_to_struct(const php_http_url_t *url, zval *strct TSRMLS_DC);
 PHP_HTTP_API char *php_http_url_to_string(const php_http_url_t *url, char **url_str, size_t *url_len, zend_bool persistent);
+PHP_HTTP_API char *php_http_url_authority_to_string(const php_http_url_t *url, char **url_str, size_t *url_len);
 PHP_HTTP_API void php_http_url_free(php_http_url_t **url);
 
-
 PHP_HTTP_API STATUS php_http_url_encode_hash(HashTable *hash, const char *pre_encoded_str, size_t pre_encoded_len, char **encoded_str, size_t *encoded_len TSRMLS_DC);
 PHP_HTTP_API STATUS php_http_url_encode_hash_ex(HashTable *hash, php_http_buffer_t *qstr, const char *arg_sep_str, size_t arg_sep_len, const char *val_sep_str, size_t val_sep_len, const char *pre_encoded_str, size_t pre_encoded_len TSRMLS_DC);
 
 static inline void php_http_url_argsep(const char **str, size_t *len TSRMLS_DC)
 {
-	if (SUCCESS != php_http_ini_entry(ZEND_STRL("arg_separator.output"), str, len, 0 TSRMLS_CC) || !*len) {
-		*str = PHP_HTTP_URL_ARGSEP;
-		*len = lenof(PHP_HTTP_URL_ARGSEP);
-	}
-}
-
-static inline php_url *php_http_url_to_php_url(php_http_url_t *url)
-{
-	php_url *purl = ecalloc(1, sizeof(*purl));
-
-	if (url->scheme)   purl->scheme   = estrdup(url->scheme);
-	if (url->pass)     purl->pass     = estrdup(url->pass);
-	if (url->user)     purl->user     = estrdup(url->user);
-	if (url->host)     purl->host     = estrdup(url->host);
-	if (url->path)     purl->path     = estrdup(url->path);
-	if (url->query)    purl->query    = estrdup(url->query);
-	if (url->fragment) purl->fragment = estrdup(url->fragment);
-
-	return purl;
+	php_http_ini_entry(ZEND_STRL("arg_separator.output"), str, len, 0 TSRMLS_CC);
 }
 
 static inline zend_bool php_http_url_is_empty(const php_http_url_t *url) {
